package session

import (
	"testing"
	tdbm "github.com/tangs-drm/go-tool/dbm"
	"ddpf/model/dbm"
	"time"
	"github.com/tangs-drm/go-tool/log"
	"fmt"
	"github.com/tangs-drm/go-tool/util"
	"github.com/tangs-drm/go-tool/http"
	ghttp "net/http"
	"io/ioutil"
)

func init() {
	// 开启服务
	SessionLog = true
	db, err := tdbm.Default("testUser:123@tcp(localhost:3306)/testdb?charset=utf8")
	if err != nil {
		panic(err)
		return
	}
	err = db.Ping()
	if err != nil {
		panic(err)
		return
	}
	// init order.sql
	var sqlConf string = "../../../order.sql"
	err = dbm.Init(sqlConf)
	if err != nil {
		panic(err)
		return
	}
}

// TestSession 对session的单元测试
func TestSession(t *testing.T) {
	log.Debug("TestSession --- START")

	fmt.Println(time.Now().Unix())
	fmt.Println(time.Now().UnixNano())

	fmt.Println(util.Now())
	fmt.Println(util.Now10())
	fmt.Println(util.Now13())

	var uid string = "u100"
	session, err := CreateSession(uid)
	if err != nil {
		t.Error(err)
		return
	}
	if len(session.Id) < 1 || session.Uid != uid {
		t.Error(session.Id, session.Uid)
		return
	}
	if session.Time > 10000000001 || session.LastTime > 10000000001 {
		t.Error(session.Time, session.LastTime)
		return
	}

	Valid_Session_Time = 5
	valid, err := CheckValidSession(session.Id)
	if err != nil {
		t.Error(err)
		return
	}
	if !valid {
		t.Error(valid)
		return
	}

	time.Sleep(10*time.Second)
	valid, err = CheckValidSession(session.Id)
	if err != nil {
		t.Error(err)
		return
	}
	if valid {
		t.Error(valid)
		return
	}

	// flushSession
	FlushSession(session.Id)
	valid, err = CheckValidSession(session.Id)
	if err != nil {
		t.Error(err)
		return
	}
	if !valid {
		t.Error(valid)
		return
	}
	log.Debug("TestSession --- END")
}

func HaloHandleFunc(w ghttp.ResponseWriter, r *ghttp.Request) {
	fmt.Println("HaloHandleFunc come in")
	w.Write([]byte("halo"))
	return
}

func RootHandleFunc(w ghttp.ResponseWriter, r *ghttp.Request) {
	fmt.Println("RootHandleFunc come in")
	w.Write([]byte("root"))
}

func TestUserFilter(t *testing.T) {
	SessionLog = true
	mux := http.NewServerMux()
	mux.FilterFunc("/usr", UserFilterFunc)
	mux.HandleFunc("/login", RootHandleFunc)
	mux.HandleFunc("/usr/halo", HaloHandleFunc)

	go func() {
		log.Error("%v", ghttp.ListenAndServe(":8023", mux))
	}()

	//time.Sleep(2 * time.Second)

	resString, err := util.HTTPGetString("http://127.0.0.1:8023/usr/halo")
	if err != nil {
		t.Error(err)
		return
	}
	fmt.Println(resString)

	// 设置session
	session, err := CreateSession("u101")
	if err != nil {
		t.Error(err)
		return
	}

	// 带上cookie的请求
	req, err := ghttp.NewRequest("GET", "http://127.0.0.1:8023/usr/halo", nil)
	if err != nil {
		t.Error(err)
		return
	}
	cookie := &ghttp.Cookie{
		Domain:"/",
		Name:"token",
		Value:session.Id,
	}
	req.AddCookie(cookie)
	client := ghttp.Client{}
	resp, err := client.Do(req)
	if err != nil {
		t.Error(err)
		return
	}
	bys, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		t.Error(err)
		return
	}
	if string(bys) != "halo" {
		t.Error(string(bys))
		return
	}

	//
	req, err = ghttp.NewRequest("GET", "http://127.0.0.1:8023/usr/halo", nil)
	if err != nil {
		t.Error(err)
		return
	}
	cookie = &ghttp.Cookie{
		Domain:"/",
		Name:"token",
		Value:util.UUID(),
	}
	req.AddCookie(cookie)
	client = ghttp.Client{}
	resp, err = client.Do(req)
	if err != nil {
		t.Error(err)
		return
	}
	bys, err = ioutil.ReadAll(resp.Body)
	if err != nil {
		t.Error(err)
		return
	}
	if string(bys) != "root" {
		t.Error(string(bys))
		return
	}
}